/****
 * sqcpsini.c: write user data on SQ-CPS diskette
 ****/

#include "sqcps.h"
#include <stdio.h>                                             
#include <io.h>
#include <fcntl.h> 

#include <malloc.h>
#include <string.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>


char logo_text [] =
"\n\
                SQ-CPS\n\n\
        initialize SQ-CPS diskette\n\n\
Copyright 1995 Software-Quelle GmbH\n\n";

char call_text [] =
"syntax:\n\
sqcpsini drive { user_data | /d:data_file } { passwd | /p:passwd_file } [ /help ]\n\n\
parameters:\n\
drive          diskette drive, for example   A:			\n\
\n\
user_data      licensing information as character string	\n\
               or						\n\
/d:data_file   data_file means the name of the file, which	\n\
               contains your licensing information (max. 4000	\n\
               bytes). All of the data in the file will be stored\n\
               on the SQ-CPS Copy Protection Diskette.		\n\
\n\
passwd         password as character string			\n\
               or						\n\
/p:passwd_file passwd_file means the name of the file, which	\n\
               contains the password (max. 4000 bytes) . All of	\n\
               the data in the file will be used as password.	\n\
\n\
/help          prints this description				\n\
\n\
";



char *	err_msg [] =
{ 
    "parameter missing after %s\n",	/* 00 */
    "invalid parameter: %s\n",		/* 01 */
    "invalid file size: %s\n",		/* 02 */
    "memory allocation error: %s\n",	/* 03 */
    "file access error: %s\n",		/* 04 */
    "drive not defined\n",		/* 05 */
    "user data not defined\n",		/* 06 */
    "password not defined\n",		/* 07 */
    "invalid drive\n",			/* 08 */
    "drive not ready\n",		/* 09 */
    "diskette error\n",			/* 10 */
    "no SQ-CPS diskette\n",		/* 11 */
    "diskette is write protected\n",		/* 12 */
};

char *	info_msg [] =
{ 
    "insert SQ-CPS diskette into drive %s\n",	/* 00 */
    "press <enter> when ready",			/* 01 */
    "SQ-CPS diskette correctly initialized\n",	/* 02 */
    "initialize another diskette? (y/n)",	/* 02 */
};




void main (argc, argv)
int argc;
char ** argv;

{
    char * arg_ptr;
    int	arg_index;
    int position_params = 0;
    int err_count = 0;
    struct _stat file_status;

    FILE * data_fd = NULL;
    FILE * passwd_fd = NULL;

    char * data_fn = 0;
    char * passwd_fn = 0;

    char * drive_ptr = 0;

    unsigned char * user_data_ptr = 0;
    short user_data_len = 0;

    unsigned char * passwd_ptr = 0;
    short passwd_len = 0;

    fprintf (stderr, logo_text);
    if (argc <= 1)
    {
	fprintf (stderr, call_text);
	exit (1);
    }

    /* evaluate parameters */
    arg_index = 1;
    for (; arg_index < argc; arg_index++)
    {
	arg_ptr = argv [arg_index];
	if (*arg_ptr == '/')
	{
	    switch (arg_ptr[1])
	    {
	    case 'd':
		/* user data file */
		if (user_data_ptr)
		{
		    fprintf (stderr, err_msg[1], arg_ptr);
		    err_count++;
		}
		if (arg_ptr[2] == ':')
		{
		    arg_ptr += 3;
		}
		else 
		{
		    arg_index++;
		    if (arg_index >= argc)
		    {
			fprintf (stderr, err_msg[0], arg_ptr);
			exit (1);
		    }
		    arg_ptr = argv [arg_index];
		}
		data_fn = arg_ptr;

		if (user_data_ptr)
		{
		    break;
		}
		do
		{
		    /* read the file */
		    if ( _stat (data_fn,
				&file_status) == 0)
		    {
			if (!(file_status.st_mode & _S_IFDIR))
			{
			    user_data_len = (int) file_status.st_size;
			    if (user_data_len > 4000)
			    {
				fprintf (stderr, err_msg[2],
					 data_fn);
				err_count++;
				break;
			    }
			    if (data_fd = fopen (data_fn,
						 "rb"))
			    {
				if (!(user_data_ptr = malloc ((int) user_data_len)))
				{
				    fprintf (stderr, err_msg[3],
					     data_fn);
				    err_count++;
				    break;
				}                
				if (fread (user_data_ptr,
					   1, user_data_len,
					   data_fd) == (unsigned int) user_data_len)
				{
				    break;
				}
				free (user_data_ptr);
				user_data_ptr = 0;
			    }
			}
		    }
		    fprintf (stderr, err_msg[4],
			     data_fn);
		    err_count++;
		    break;

		} while (0);
		break;

	    case 'p':
		/* password file */
		if (passwd_ptr)
		{
		    fprintf (stderr, err_msg[1], arg_ptr);
		    err_count++;
		    break;
		}
		if (arg_ptr[2] == ':')
		{
		    arg_ptr += 3;
		}
		else 
		{
		    arg_index++;
		    if (arg_index >= argc)
		    {
			fprintf (stderr, err_msg[0], arg_ptr);
			err_count++;
			break;
		    }
		    arg_ptr = argv [arg_index];
		}

		if (passwd_ptr)
		{
		    break;
		}
		passwd_fn = arg_ptr;
		do
		{
		    /* read the file */
		    if ( _stat (passwd_fn,
				&file_status) == 0)
		    {
			if (!(file_status.st_mode & _S_IFDIR))
			{
			    passwd_len = (int) file_status.st_size;
			    if (passwd_len > 4000)
			    {
				fprintf (stderr, err_msg[2],
					 passwd_fn);
				err_count++;
				break;
			    }
			    if (passwd_fd = fopen (passwd_fn,
						   "rb"))
			    {
				if (!(passwd_ptr = malloc ((int) passwd_len + 1)))
				{
				    fprintf (stderr, err_msg[3],
					     passwd_fn);
				    err_count++;
				    break;
				}                
				if (fread (passwd_ptr,
					   1, passwd_len,
					   passwd_fd) == (unsigned int) passwd_len)
				{
				    passwd_ptr[passwd_len] = '\0';
				    break;
				}
			    }
			}
		    }
		    fprintf (stderr, err_msg[4],
			     passwd_fn);
		    err_count++;
		    break;

		} while (0);
		break;

	    case 'h':
		/* help? */
		fprintf (stderr, call_text);
		exit (1);

	    default:
		fprintf (stderr, err_msg[1],
			 arg_ptr);
		err_count++;
		break;
	    }
	    continue;
	}
	switch (position_params)
	{
	case 0:
	    /* drive */
	    if (!(drive_ptr))
	    {
		drive_ptr = arg_ptr;
		break;
	    }
	    position_params ++;

	case 1:
	    /* user data */
	    if (!(user_data_ptr))
	    {
		user_data_ptr = arg_ptr;
		user_data_len = strlen (user_data_ptr);
		break;
	    }
	    position_params ++;

	case 2:
	    /* password */
	    if (!(passwd_ptr))
	    {
		passwd_ptr = arg_ptr;
		break;
	    }
	    position_params ++;

	default:
	    fprintf (stderr, err_msg[1],
		     arg_ptr);
	    err_count++;
	    break;
	}
	position_params ++;
    }
    if (!(drive_ptr))
    {
	fprintf (stderr, err_msg[5]);
	err_count++;
    }
    if (err_count)
    {
	fprintf (stderr, call_text);
	exit (1);
    }
    for (;;)
    {
	char input_string[20];
	fprintf (stderr, info_msg[0], drive_ptr);
	fprintf (stderr, info_msg[1]);
	if (fgets (input_string, sizeof (input_string), stdin))
	{
	    fprintf (stderr, "\n");
	    /* Puffer schreiben */
	    switch (sqcpswr (drive_ptr,
			    user_data_ptr, user_data_len,
			    passwd_ptr))
	    {
	    case 0:
		fprintf (stderr, info_msg[2]);
		break;

	    case 1:
		fprintf (stderr, err_msg[8]);
		break;

	    case 2:
		fprintf (stderr, err_msg[9]);
		break;

	    case 3:
		fprintf (stderr, err_msg[10]);
		break;
		
	    case 4:
		fprintf (stderr, err_msg[11]);
		break;

	    case 5:
		fprintf (stderr, err_msg[12]);
		break;
		
	    }        
	}
	for (;;)
	{
	    fprintf (stderr, info_msg[3]);
	    if (fgets (input_string, sizeof (input_string), stdin))
	    {
		fprintf (stderr, "\n");
		switch (input_string[0])
		{
		case 'Y':
		case 'y':
		    break;

		case 'N':
		case 'n':
		    exit (0);
		default:
		    continue;
		}
		break;
	    }
	}
    }
}